#!/usr/bin/env python3
# molmass_web.py

# Copyright (c) 2005-2021, Christoph Gohlke
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""Molecular Mass Calculator - A simple web interface for molmass.py.

Run ``python -m molmass --web`` to execute the script in a local web server.

Revisions
---------
2021.6.18
    Link to source code on GitHub.
    Fix failures on WSL2 (#9).
2020.1.1
    Remove support for Python 2.7 and 3.5.
2018.8.15
    Move module into molmass package.
2018.5.29
    Use CSS flex layout.
    Separate styles from content.
    Run http server from this file's directory.
2018.5.25
    Backwards incompatible changes:
    Rename to molmass_web.py.
    Accept Flask request.args.
    Style and template changes.
    Fix regular expressions.
2018.2.6
    Style and docstring fixes.
    Escape error messages.
2005.x.x
    Initial release.

"""

__version__ = '2021.6.18'

import os
import re
import sys
from html import escape

try:
    from . import molmass
except ImportError:
    import molmass


PAGE = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<style type="text/css">
html {{font-size:16px}}
body {{min-width:576px}}
div.header {{display:flex;flex-wrap:wrap;align-items:baseline}}
div.header a {{font-weight:bold;text-decoration:none}}
h1 {{margin:0;padding:0 0.5em 0 0;}}
h1 a {{text-decoration:none;color:#000000}}
form {{display:flex;flex-wrap:nowrap;background-color:#eeeeee;
border:1px solid #aaaaaa;padding:1em;}}
input {{margin-left:0.5em;min-width:4.5em;}}
input[type=text] {{width:100%}}
div.formula {{flex:1 1 100%;min-width:2em;margin-right:1em}}
div.buttons {{white-space:nowrap}}
</style>
{heads}
<meta name="generator" content="molmass_web.py" />
<meta name="robots" content="noarchive" />
<meta name="format-detection" content="telephone=no" />
<meta name="viewport" content="width=608px" />
<title>Molecular Mass Calculator - Christoph Gohlke</title>
</head>
<body>
<div class="header">
<h1><a href="{url}"
title="Molecular Mass Calculator version {version} by Christoph Gohlke"
>Molecular Mass Calculator</a></h1>
<p>by <a href="https://www.lfd.uci.edu/~gohlke/">Christoph Gohlke</a></p>
</div>
<form id="molmass" method="get" action="">
<div><label for="q"><strong>Formula:</strong></label></div>
<div class="formula">
<input type="text" name="q" id="q" value="{formula}" />
</div>
<div class="buttons">
<input type="submit" id="a" value="Submit" />
<input type="reset" value="Reset" onclick="window.location='{url}'"/>
</div>
</form>
<div class="content">
{content}
</div>
</body>
</html>"""

HELP = """<p>This web application calculates the molecular mass (average,
monoisotopic, and nominal), the elemental composition, and the mass
distribution spectrum of a molecule given by its chemical formula, relative
element weights, or sequence.</p>
<p>Calculations are based on the
<a href="?q=isotopes" rel="nofollow">isotopic composition of the elements</a>.
Mass deficiency due to chemical bonding is not considered.
</p>
<h3>Examples</h3>
<ul>
<li>Simple chemical formulas:
<a href="?q=H2O" rel="nofollow">H2O</a> or
<a href="?q=CH3COOH" rel="nofollow">CH3COOH</a>
</li>
<li>Specific isotopes:
<a href="?q=D2O" rel="nofollow">D2O</a> or
<a href="?q=[30Si]3O2" rel="nofollow">[30Si]3O2</a>
</li>
<li><a href="?q=groups" rel="nofollow">Abbreviations</a> of chemical groups:
<a href="?q=EtOH" rel="nofollow">EtOH</a> or
<a href="?q=PhOH" rel="nofollow">PhOH</a>
</li>
<li>Simple arithmetic:
<a href="?q=(COOH)2" rel="nofollow">(COOH)2</a> or
<a href="?q=CuSO4.5H2O" rel="nofollow">CuSO4.5H2O</a>
</li>
<li>Relative element weights:
<a href="?q=O: 0.26, 30Si: 0.74" rel="nofollow">O: 0.26, 30Si: 0.74</a>
</li>
<li>Nucleotide sequences:
<a href="?q=CGCGAATTCGCG" rel="nofollow">CGCGAATTCGCG</a> or
<a href="?q=dsrna(CCUU)" rel="nofollow">dsrna(CCUU)</a>
</li>
<li>Peptide sequences:
<a href="?q=MDRGEQGLLK" rel="nofollow">MDRGEQGLLK</a> or
<a href="?q=peptide(CPK)" rel="nofollow">peptide(CPK)</a>
</li>
</ul>
<p>Formulas are case sensitive and &#8217;+&#8217; denotes
the arithmetic operator, not an ion charge.</p>
<div class="disclaimer">
<h3>Disclaimer</h3>
<p>Because this service is provided free of charge, there is no
warranty for the service, to the extent permitted by applicable law.
The service is provided &quot;as is&quot; without warranty of any kind,
either expressed or implied, including, but not limited to, the implied
warranties of merchantability and fitness for a particular purpose.
The entire risk as to the quality and performance is with you.</p>
</div>
<div class="about">
<h3>About</h3>
<p>Molecular Mass Calculator version {version} by
<a href="https://www.lfd.uci.edu/~gohlke/">Christoph Gohlke</a>.
Source code is available on
<a href="https://github.com/cgohlke/molmass">GitHub</a>.</p>
</div>"""


def response(form, url, template=PAGE, help=HELP, heads=''):
    """Return HTML document from submitted form data."""
    formula = form.get('q', '')
    if not formula:
        formula = ''
    if formula == 'groups':
        formula = ''
        content = groups()
    elif formula == 'isotopes':
        formula = ''
        content = isotopes()
    elif formula:
        content = analyze(formula[:100])
    else:
        content = help.format(version=__version__)
    if formula:
        formula = escape(formula, True)
    return template.format(
        formula=formula,
        url=url,
        version=__version__,
        content=content,
        heads=heads.strip(),
    )


def analyze(formula, maxatoms=250):
    """Return analysis of formula as HTML string."""

    def html(formula):
        """Return formula as HTML string."""
        formula = re.sub(
            r'\[(\d+)([A-Za-z]{1,2})\]', r'<sup>\1</sup>\2', formula
        )
        formula = re.sub(r'([A-Za-z]{1,2})(\d+)', r'\1<sub>\2</sub>', formula)
        return formula

    result = []
    try:
        f = molmass.Formula(formula)
        result.append(
            f'<p><strong>Hill notation</strong>: {html(f.formula)}</p>'
        )
        if f.formula != f.empirical:
            result.append(
                f'<p><strong>Empirical formula</strong>:'
                f' {html(f.empirical)}</p>'
            )

        prec = molmass.precision_digits(f.mass, 8)
        if f.mass != f.isotope.mass:
            result.append(
                f'<p><strong>Average mass</strong>: {f.mass:.{prec}f}</p>'
            )
        result.extend(
            (
                f'<p><strong>Monoisotopic mass</strong>:'
                f' {f.isotope.mass:.{prec}f}</p>',
                f'<p><strong>Nominal mass</strong>:'
                f' {f.isotope.massnumber}</p>',
            )
        )

        c = f.composition()
        if len(c) > 1:
            result.extend(
                (
                    '<h3>Elemental Composition</h3>'
                    '<table border="0" cellpadding="2">',
                    '<tr>',
                    '<th scope="col" align="left">Element</th>',
                    '<th scope="col" align="right">Number</th>',
                    '<th scope="col" align="right">Relative mass</th>',
                    '<th scope="col" align="right">Fraction %</th>',
                    '</tr>',
                )
            )

            for symbol, count, mass, fraction in c:
                symbol = re.sub(r'^(\d+)(.+)', r'<sup>\1</sup>\2', symbol)
                result.extend(
                    (
                        f'<tr><td>{symbol}</td>',
                        f'<td align="center">{count}</td>',
                        f'<td align="right">{mass:.{prec}f}</td>',
                        f'<td align="right">{fraction * 100:.4f}</td></tr>',
                    )
                )

            count, mass, fraction = c.total
            result.extend(
                (
                    '<tr><td><em>Total</em></td>',
                    f'<td align="center"><em>{count}</em></td>',
                    f'<td align="right"><em>{mass:.{prec}f}</em></td>',
                    f'<td align="right"><em>{fraction * 100:.4f}</em></td>',
                    '</tr>',
                    '</table>',
                )
            )

        if f.atoms < maxatoms:
            s = f.spectrum()
            if len(s) > 1:
                norm = 100.0 / s.peak[1]
                result.extend(
                    (
                        '<h3>Mass Distribution</h3>',
                        '<p><strong>Most abundant mass</strong>: ',
                        f'{s.peak[0]:.{prec}f} ({s.peak[1] * 100:.3f}%)</p>',
                        f'<p><strong>Mean mass</strong>: {s.mean:.{prec}}</p>',
                        '<table border="0" cellpadding="2">',
                        '<tr>',
                        '<th scope="col" align="left">Relative mass</th>',
                        '<th scope="col" align="right">Fraction %</th>',
                        '<th scope="col" align="right">Intensity</th>',
                        '</tr>',
                    )
                )
                for mass, fraction in s.values():
                    result.extend(
                        (
                            f'<tr><td>{mass:.{prec}f}</td>',
                            f'<td align="right">{fraction*100.:.6}</td>',
                            f'<td align="right">{fraction*norm:.6}</td></tr>',
                        )
                    )
                result.append('</table>')

    except Exception as exc:
        exc = str(exc).splitlines()
        text = exc[0][0].upper() + exc[0][1:]
        details = '\n'.join(exc[1:])
        result.append(
            '<h2>Error</h2><p>{}</p><pre>{}</pre>'.format(
                escape(text, True), escape(details, True)
            )
        )

    return '\n'.join(result)


def isotopes():
    """Return HTML table of isotope masses and abundances."""
    template = """
    <h3>Isotopic Composition of the Elements</h3>
    <table border="0" cellpadding="2">
    <tr>
    <th align="left" scope="col">Element/Isotope</th>
    <th align="right" scope="col">Relative mass</th>
    <th align="right" scope="col">Abundance</th>
    </tr>
    {rows}
    </table>"""
    rows = []
    for ele in molmass.ELEMENTS:
        rows.extend(
            (
                f'<tr><td><em>{ele.name}</em></td>',
                f'<td align="right">{ele.mass:12.8f}</td></tr>',
            )
        )
        for massnumber in sorted(ele.isotopes):
            iso = ele.isotopes[massnumber]
            rows.extend(
                (
                    '<tr>'
                    f'<td align="right"><sup>{massnumber}</sup>{ele.symbol}'
                    f'</td>',
                    f'<td align="right">{iso.mass:.8f}</td>',
                    f'<td align="right">{iso.abundance * 100.:.8f}</td>',
                    '</tr>',
                )
            )
    return template.format(rows='\n'.join(rows))


def groups():
    """Return HTML table of chemical groups."""
    template = """
    <h3>{title}</h3>
    <table border="0" cellpadding="2">
    <tr>
    <th align="left" scope="col">Name</th>
    <th align="left" scope="col">Formula</th>
    </tr>
    {rows}
    </table>"""
    result = []
    for group, title in (
        (molmass.GROUPS, 'Chemical Groups'),
        (molmass.AMINOACIDS, 'Amino Acids'),
        (molmass.DEOXYNUCLEOTIDES, 'Deoxynucleotides'),
        (molmass.NUCLEOTIDES, 'Nucleotides'),
    ):
        rows = []
        for key in sorted(group):
            value = group[key]
            if isinstance(value, str):
                rows.append(f'<tr><td>{key}</td><td>{value}</td></tr>')
        result.append(template.format(title=title, rows='\n'.join(rows)))
    return '\n'.join(result)


def main(url='http://localhost:9000/{}'):
    """Run web application in local web server."""
    import cgi
    import cgitb

    cgitb.enable()

    dirname, filename = os.path.split(__file__)
    if filename[-1] != 'y':
        filename = filename[:-1]  # don't use .pyc or .pyo
    url = url.format(filename)
    if dirname:
        os.chdir(dirname)

    if os.getenv('SERVER_NAME'):
        print('Content-type: text/html\n\n')
        request = cgi.FieldStorage()
        request.get = request.getfirst
        print(response(request, url))
    else:
        import webbrowser
        from urllib.parse import urlparse
        from http.server import HTTPServer, CGIHTTPRequestHandler

        def is_cgi(self):
            """Monkey patch for CGIHTTPRequestHandler.is_cgi."""
            if filename in self.path:
                self.cgi_info = '', self.path[1:]
                return True
            return False

        CGIHTTPRequestHandler.is_cgi = is_cgi
        print('Serving CGI script at', url)
        webbrowser.open(url)
        url = urlparse(url)
        HTTPServer(
            (url.hostname, url.port), CGIHTTPRequestHandler
        ).serve_forever()


if __name__ == '__main__':
    sys.exit(main())



# elements_gui.py

# Copyright (c) 2005-2021, Christoph Gohlke
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""Periodic Table of Elements - A user interface for elements.py.

Revisions
---------
2021.6.18
    Remove support for Python 3.6 (NEP 29).
    Fix symbol size on WSL2.
2020.6.10
    Support wxPython 4.1.
2019.1.1
    Require Python 3 and wxPython 4.
2018.8.15
    Move module into molmass package.
2018.5.25
    Style fixes.

"""

import os
import sys
import math
import io

import wx
from wx.lib import fancytext, buttons, rcsizer

if __package__:
    from .elements import ELEMENTS, SERIES
else:
    from elements import ELEMENTS, SERIES


class MainApp(wx.App):
    """Main application."""

    name = 'Periodic Table of Elements'
    version = '2021.6.18'
    website = 'https://www.lfd.uci.edu/~gohlke/'
    copyright = (
        'Christoph Gohlke\n'
        'Laboratory for Fluorescence Dynamics\n'
        'University of California, Irvine'
    )
    icon = 'icon.png'
    try:
        icon = os.path.join(os.path.dirname(__file__), icon)
    except Exception:
        pass

    def OnInit(self):
        wx.LogNull()
        # wx.InitAllImageHandlers()
        mainframe = MainFrame(None, -1)
        self.SetTopWindow(mainframe)
        if wx.Platform != '__WXMAC__':
            mainframe.Centre()
        mainframe.ApplyLayout(False)
        mainframe.Show()
        return 1


class MainFrame(wx.Frame):
    """Main application window."""

    def __init__(self, *args, **kwds):
        kwds['style'] = (
            wx.DEFAULT_DIALOG_STYLE | wx.MINIMIZE_BOX | wx.TAB_TRAVERSAL
        )
        wx.Frame.__init__(self, *args, **kwds)
        self.selected = -1

        self.SetTitle(MainApp.name)
        if os.path.exists(MainApp.icon):
            icon = wx.Icon()
            icon.CopyFromBitmap(wx.Bitmap(MainApp.icon, wx.BITMAP_TYPE_ANY))
            self.SetIcon(icon)
        self.SetBackgroundColour(
            wx.SystemSettings.GetColour(wx.SYS_COLOUR_3DFACE)
        )

        # create menu
        self.menu = wx.MenuBar()
        self.SetMenuBar(self.menu)
        menu = wx.Menu()
        menu.Append(wx.ID_EXIT, 'Exit', 'Exit the application', wx.ITEM_NORMAL)
        self.menu.Append(menu, 'File')
        menu = wx.Menu()
        menu.Append(
            wx.ID_COPY,
            'Copy\tCtrl+C',
            'Copy selected element to the clipboard',
            wx.ITEM_NORMAL,
        )
        self.menu.Append(menu, 'Edit')
        menu = wx.Menu()
        menu.Append(
            wx.ID_VIEW_DETAILS,
            'Details',
            'Show or hide details',
            wx.ITEM_CHECK,
        )
        self.menu.Append(menu, 'View')
        menu = wx.Menu()
        menu.Append(
            wx.ID_ABOUT,
            'About...',
            'Display information about the program',
            wx.ITEM_NORMAL,
        )
        self.menu.Append(menu, 'Help')

        # create panels and controls
        self.notebook = wx.Notebook(self, -1, style=0)
        self.description = DecriptionPanel(self.notebook, -1)
        self.details = DetailsPanel(self.notebook, -1)
        self.table = PeriodicPanel(self, -1)
        self.disclose = DisclosureCtrl(self.table, -1, '')
        self.table.AddCtrl(self.disclose)
        self.notebook.AddPage(self.description, 'Description')
        self.notebook.AddPage(self.details, 'Properties')

        # event bindings
        self.Bind(wx.EVT_MENU, self.OnExit, id=wx.ID_EXIT)
        self.Bind(wx.EVT_MENU, self.OnCopy, id=wx.ID_COPY)
        self.Bind(wx.EVT_MENU, self.OnDetails, id=wx.ID_VIEW_DETAILS)
        self.Bind(wx.EVT_MENU, self.OnAbout, id=wx.ID_ABOUT)
        self.Bind(EVT_ELE_CHANGED, self.OnSelect, self.table)
        self.Bind(EVT_ELE_CHANGED, self.OnSelect, self.details)
        self.Bind(wx.EVT_BUTTON, self.OnDetails, self.disclose)

        # create sizers
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(
            self.table,
            1,
            wx.LEFT | wx.TOP | wx.RIGHT | wx.EXPAND | wx.ADJUST_MINSIZE,
            # wx.ALIGN_CENTER_HORIZONTAL |
            BORDER - 5,
        )
        self.sizer.Add((BORDER, BORDER))
        self.sizer.Add(
            self.notebook,
            0,
            wx.LEFT | wx.RIGHT | wx.BOTTOM | wx.EXPAND |
            # wx.ALIGN_CENTER_HORIZONTAL | wx.ALIGN_CENTER_VERTICAL |
            wx.ADJUST_MINSIZE,
            BORDER,
        )

        self.notebook.SetSelection(1)
        self.SetAutoLayout(True)
        self.SetSizerAndFit(self.sizer, True)
        self.sizer.SetSizeHints(self)
        self.ApplyLayout(True)
        self.SetSelection(0)

    def ApplyLayout(self, show=False):
        self.show_info = show
        self.sizer.Show(self.notebook, show, True)
        self.SetAutoLayout(True)
        self.SetSizerAndFit(self.sizer, True)
        self.sizer.SetSizeHints(self)
        self.Layout()
        self.menu.Check(wx.ID_VIEW_DETAILS, show)
        self.disclose.SetToggle(self.show_info)

    def OnDetails(self, evt):
        self.ApplyLayout(not self.show_info)

    def OnEraseBackground(self, event):
        pass

    def OnCopy(self, evt):
        dobj = wx.TextDataObject()
        dobj.SetText(repr(ELEMENTS[self.selected + 1]))
        if wx.TheClipboard.Open():
            wx.TheClipboard.SetData(dobj)
            wx.TheClipboard.Close()

    def OnAbout(self, evt):
        import wx.adv

        info = wx.adv.AboutDialogInfo()
        info.SetName(MainApp.icon)
        info.SetName(MainApp.name)
        info.SetVersion(MainApp.version)
        info.SetCopyright(MainApp.copyright)
        info.SetWebSite(MainApp.website)
        wx.adv.AboutBox(info)

    def OnWebsite(self, evt):
        import webbrowser

        webbrowser.open(MainApp.website)

    def OnWikipedia(self, evt):
        import webbrowser

        ele = ELEMENTS[self.selected].name
        webbrowser.open(f'https://en.wikipedia.org/wiki/{ele}', 1)

    def OnWebElements(self, evt):
        import webbrowser

        ele = ELEMENTS[self.selected].name.lower()
        webbrowser.open(f'https://www.webelements.com/{ele}/', 1)

    def OnSelect(self, evt):
        self.SetSelection(evt.GetSelection())

    def SetSelection(self, select):
        """Set active element."""
        if self.selected != select:
            self.selected = select
            self.description.SetSelection(select)
            self.table.SetSelection(select)
            self.details.SetSelection(select)

    def OnExit(self, evt):
        self.Close()
        if __name__ == "__main__":
            sys.exit(0)
        else:
            return self.selected


class PeriodicPanel(wx.Panel):
    """Periodic table of elements panel."""

    layout = """
        .  1  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  18 .
        1  H  2  .  .  .  .  .  .  .  .  .  .  13 14 15 16 17 He .
        2  Li Be .  .  .  .  .  .  .  .  .  .  B  C  N  O  F  Ne .
        3  Na Mg 3  4  5  6  7  8  9  10 11 12 Al Si P  S  Cl Ar .
        4  K  Ca Sc Ti V  Cr Mn Fe Co Ni Cu Zn Ga Ge As Se Br Kr .
        5  Rb Sr Y  Zr Nb Mo Tc Ru Rh Pd Ag Cd In Sn Sb Te I  Xe .
        6  Cs Ba *  Hf Ta W  Re Os Ir Pt Au Hg Tl Pb Bi Po At Rn .
        7  Fr Ra ** Rf Db Sg Bh Hs Mt .  .  .  .  .  .  .  .  .  .
        .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .  .
        .  .  .  *  La Ce Pr Nd Pm Sm Eu Gd Tb Dy Ho Er Tm Yb Lu .
        .  .  .  ** Ac Th Pa U  Np Pu Am Cm Bk Cf Es Fm Md No Lr .
    """

    def __init__(self, *args, **kwds):
        kwds['style'] = wx.TAB_TRAVERSAL
        wx.Panel.__init__(self, *args, **kwds)
        rows = len(self.layout.splitlines()) - 2
        cols = len(self.layout.splitlines()[1].split())
        self.sizer = wx.FlexGridSizer(rows, cols, 0, 0)
        self.buttons = list(range(0, len(ELEMENTS)))
        self.selected = -1

        self.info = ElementPanel(self, -1, pos=(0, 0))

        # create element buttons
        buttonsize = int(math.ceil((self.info.Size[0] + 4) / 9.0))
        if buttonsize < 30:
            buttonsize = 30
        for row in self.layout.splitlines()[1:-1]:
            for col in row.split():
                if col == '.':
                    self.sizer.Add((SPACER, SPACER))
                elif col[0] in '123456789*':
                    static = wx.StaticText(
                        self, -1, col, style=wx.ALIGN_CENTER | wx.ALIGN_BOTTOM
                    )
                    self.sizer.Add(
                        static,
                        0,
                        wx.ALL
                        | wx.ALIGN_CENTER_HORIZONTAL
                        | wx.ALIGN_CENTER_VERTICAL
                        | wx.FIXED_MINSIZE,
                        SPACER // 2,
                    )
                else:
                    ele = ELEMENTS[col]
                    button = ElementButton(
                        self,
                        ele.number + 100,
                        ele.symbol,
                        size=(buttonsize, buttonsize),
                    )
                    self.buttons[ele.number - 1] = button
                    button.SetBezelWidth(1)
                    button.SetToolTip(ele.name)
                    col = COLORS[ele.series]
                    button.SetButtonColour(wx.Colour(col[0], col[1], col[2]))
                    self.sizer.Add(
                        button,
                        0,
                        wx.LEFT
                        | wx.BOTTOM
                        | wx.FIXED_MINSIZE
                        | wx.ALIGN_CENTER_HORIZONTAL
                        | wx.ALIGN_CENTER_VERTICAL,
                        0,
                    )
                    self.Bind(wx.EVT_BUTTON, self.OnSelect, button)

        self.SetAutoLayout(True)
        self.SetSizer(self.sizer)
        self.sizer.SetSizeHints(self)
        self.sizer.Fit(self)

        # position element info panel
        cw = self.sizer.GetColWidths()
        rh = self.sizer.GetRowHeights()
        self.info.Move((sum(cw[:3]) + cw[3] // 2, (rh[0] - SPACER) // 2 - 1))

        # legend of chemical series
        self.legendpos = (sum(cw[:13]), (rh[0] - SPACER) // 2 - 1)
        self.legendsize = (sum(cw[13:17]) + cw[17] // 2, -1)
        self.legend = wx.StaticText(
            self,
            -1,
            ' Alkaline earth metals ',
            style=wx.ALIGN_CENTER,
            pos=self.legendpos,
            size=self.legendsize,
        )
        self.legend.SetToolTip('Chemical series')

        # blinking element button
        self.highlight = False
        self.timer = wx.Timer(self)
        self.timer.Start(750)
        self.Bind(wx.EVT_TIMER, self.OnTimer)

    def AddCtrl(self, ctrl, pos=200):
        self.sizer.Remove(pos)
        self.sizer.Insert(
            pos,
            ctrl,
            0,
            wx.ALL
            | wx.ALIGN_CENTER_HORIZONTAL
            | wx.ALIGN_BOTTOM
            | wx.FIXED_MINSIZE,
            0,
        )
        self.Layout()

    def OnTimer(self, evt):
        button = self.buttons[self.selected]
        button.SetToggle(button.up)

    def OnSelect(self, evt):
        self.GetParent().SetSelection(evt.GetId() - 101)

    def SetSelection(self, select):
        """Set active element."""
        if self.selected == select:
            return
        # reset old selection
        self.buttons[self.selected].SetToggle(False)
        # highlight new selection
        self.selected = select
        self.buttons[select].SetToggle(True)
        ele = ELEMENTS[select + 1]
        col = COLORS[ele.series]
        self.legend.SetBackgroundColour(wx.Colour(col[0], col[1], col[2]))
        self.legend.SetLabel(SERIES[ele.series])
        self.legend.Move(self.legendpos)
        self.legend.SetSize(self.legendsize)
        self.info.SetSelection(select)


class ElementPanel(wx.Panel):
    """Element information panel."""

    def __init__(self, *args, **kwds):
        kwds['style'] = wx.NO_BORDER | wx.TAB_TRAVERSAL
        wx.Panel.__init__(self, *args, **kwds)
        self.selected = -1

        # create controls
        self.number = wx.StaticText(self, -1, '808', style=wx.ALIGN_RIGHT)
        self.position = wx.StaticText(
            self, -1, '6, 88, 9', style=wx.ALIGN_LEFT
        )
        self.symbol = wx.StaticText(
            self, -1, 'Mm', style=wx.ALIGN_CENTER_HORIZONTAL
        )
        self.name = wx.StaticText(
            self, -1, 'Praseodymium ', style=wx.ALIGN_CENTER_HORIZONTAL
        )
        self.mass = wx.StaticText(
            self, -1, '123.4567890 ', style=wx.ALIGN_CENTER_HORIZONTAL
        )
        self.massnumber = wx.StaticText(
            self, -1, '123 A ', style=wx.ALIGN_RIGHT
        )
        self.protons = wx.StaticText(self, -1, '123 P ', style=wx.ALIGN_RIGHT)
        self.neutrons = wx.StaticText(self, -1, '123 N ', style=wx.ALIGN_RIGHT)
        self.electrons = wx.StaticText(
            self, -1, '123 e ', style=wx.ALIGN_RIGHT
        )
        self.eleshell = wx.StaticText(
            self, -1, '2, 8, 18, 32, 32, 15, 2', style=wx.ALIGN_LEFT
        )
        self.eleconfig = StaticFancyText(
            self,
            -1,
            '[Xe] 4f<sup>14</sup> 5d<sup>10</sup>'
            ' 6s<sup>2</sup> 6p<sup>6</sup> ',
            style=wx.ALIGN_LEFT,
        )
        self.oxistates = wx.StaticText(self, -1, '1*, 2, 3, 4, 5, 6, -7 ')
        self.atmrad = wx.StaticText(self, -1, '1.234 A ', style=wx.ALIGN_RIGHT)
        self.ionpot = wx.StaticText(self, -1, '123.4567890 eV ')
        self.eleneg = wx.StaticText(self, -1, '123.45678 ')

        # set control properties
        font = self.GetFont()
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        self.number.SetFont(font)
        self.name.SetFont(font)
        font.SetPointSize(int(font.GetPointSize() * 1.9))
        self.symbol.SetFont(font)

        self.number.SetToolTip('Atomic number')
        self.position.SetToolTip('Group, Period, Block')
        self.symbol.SetToolTip('Symbol')
        self.name.SetToolTip('Name')
        self.mass.SetToolTip('Relative atomic mass')
        self.eleshell.SetToolTip('Electrons per shell')
        self.massnumber.SetToolTip('Mass number (most abundant isotope)')
        self.protons.SetToolTip('Protons')
        self.neutrons.SetToolTip('Neutrons (most abundant isotope)')
        self.electrons.SetToolTip('Electrons')
        self.eleconfig.SetToolTip('Electron configuration')
        self.oxistates.SetToolTip('Oxidation states')
        self.atmrad.SetToolTip('Atomic radius')
        self.ionpot.SetToolTip('Ionization potentials')
        self.eleneg.SetToolTip('Electronegativity')

        # layout
        sizer = rcsizer.RowColSizer()
        sizer.col_w = SPACER
        sizer.row_h = SPACER
        sizer.Add(
            self.number,
            row=0,
            col=0,
            flag=wx.ALIGN_CENTER_HORIZONTAL | wx.FIXED_MINSIZE,
        )
        sizer.Add(
            self.position, row=0, col=1, flag=wx.ALIGN_LEFT | wx.FIXED_MINSIZE
        )
        sizer.Add(
            self.symbol,
            row=1,
            col=0,
            rowspan=2,
            colspan=2,
            flag=(
                wx.ALIGN_CENTER_HORIZONTAL
                | wx.ALIGN_CENTER_VERTICAL
                | wx.EXPAND
                | wx.ALL
            ),
        )
        sizer.Add(
            self.name,
            row=3,
            col=0,
            colspan=2,
            flag=wx.ALIGN_CENTER_HORIZONTAL | wx.FIXED_MINSIZE,
        )
        sizer.Add(
            self.mass,
            row=4,
            col=0,
            colspan=2,
            flag=wx.ALIGN_CENTER_HORIZONTAL | wx.FIXED_MINSIZE,
        )
        sizer.Add(
            self.massnumber,
            row=0,
            col=2,
            flag=wx.ALIGN_RIGHT | wx.FIXED_MINSIZE,
        )
        sizer.Add(
            self.protons, row=1, col=2, flag=wx.ALIGN_RIGHT | wx.FIXED_MINSIZE
        )
        sizer.Add(
            self.neutrons, row=2, col=2, flag=wx.ALIGN_RIGHT | wx.FIXED_MINSIZE
        )
        sizer.Add(
            self.electrons,
            row=3,
            col=2,
            flag=wx.ALIGN_RIGHT | wx.FIXED_MINSIZE,
        )
        sizer.Add(
            self.atmrad, row=4, col=2, flag=wx.ALIGN_RIGHT | wx.FIXED_MINSIZE
        )
        sizer.Add(self.eleconfig, row=0, col=4, flag=wx.ADJUST_MINSIZE)
        sizer.Add(self.eleshell, row=1, col=4, flag=wx.ADJUST_MINSIZE)
        sizer.Add(self.oxistates, row=2, col=4, flag=wx.ADJUST_MINSIZE)
        sizer.Add(self.ionpot, row=3, col=4, flag=wx.ADJUST_MINSIZE)
        sizer.Add(self.eleneg, row=4, col=4, flag=wx.ADJUST_MINSIZE)

        self.SetAutoLayout(True)
        self.SetSizer(sizer)
        sizer.Fit(self)
        sizer.SetSizeHints(self)
        self.Layout()

    def SetSelection(self, select):
        """Set active element."""
        if self.selected == select:
            return
        self.Freeze()
        self.selected = select
        ele = ELEMENTS[select + 1]

        self.number.SetLabel(f'{ele.number}')
        self.position.SetLabel(f'{ele.group}, {ele.period}, {ele.block}')
        self.mass.SetLabel(f'{ele.mass:.10g}')
        self.eleshell.SetLabel(', '.join(f'{i}' for i in ele.eleshells))
        self.massnumber.SetLabel(f'{ele.nominalmass} A ')
        self.protons.SetLabel(f'{ele.protons} P ')
        self.neutrons.SetLabel(f'{ele.neutrons} N ')
        self.electrons.SetLabel(f'{ele.electrons} e ')
        self.oxistates.SetLabel(ele.oxistates)
        self.atmrad.SetLabel(f'{ele.atmrad:.10g} \xc5 ' if ele.atmrad else '')
        self.eleneg.SetLabel(f'{ele.eleneg:.10g} ' if ele.eleneg else '')
        self.ionpot.SetLabel(
            f'{ele.ionenergy[0]:.10g} eV' if ele.ionenergy else ''
        )
        self.symbol.SetLabel(ele.symbol)
        self.name.SetLabel(ele.name)

        label = []
        for orb in ele.eleconfig.split():
            if not orb.startswith('[') and len(orb) > 2:
                orb = orb[:2] + '<sup>' + orb[2:] + '</sup>'
            label.append(orb)
        label.append('<sup> </sup>')  # fix ADJUST_MINSIZE
        self.eleconfig.SetLabel(' '.join(label))

        self.Thaw()
        self.Layout()


class DetailsPanel(wx.Panel):
    """Element details panel."""

    def __init__(self, *args, **kwds):
        kwds['style'] = wx.NO_BORDER | wx.TAB_TRAVERSAL
        wx.Panel.__init__(self, *args, **kwds)
        self.selected = -1

        # create controls
        self.names = LabeledCtrl(
            self,
            wx.ComboBox,
            'Element Name',
            choices=[p.name for p in ELEMENTS],
            style=wx.CB_READONLY | wx.CB_SORT,
            size=(1, -1),
        )
        self.symbols = LabeledCtrl(
            self,
            wx.ComboBox,
            'Symbol',
            '',
            choices=[p.symbol for p in ELEMENTS],
            style=wx.CB_READONLY | wx.CB_SORT,
            size=(1, -1),
        )
        self.numbers = LabeledCtrl(
            self,
            wx.ComboBox,
            'Number',
            choices=[f'{p.number}' for p in ELEMENTS],
            style=wx.CB_READONLY,
            size=(1, -1),
        )
        self.mass = LabeledCtrl(
            self,
            wx.ComboBox,
            'Relative Atomic Mass',
            choices=[f'{p.mass:.10g}' for p in ELEMENTS],
            style=wx.CB_READONLY,
            size=(1, -1),
        )
        self.atmrad = LabeledCtrl(
            self,
            wx.ComboBox,
            'Atomic Radius (\xc5)',
            choices=[f'{p.atmrad:.10g}' for p in ELEMENTS],
            style=wx.CB_READONLY,
            size=(1, -1),
        )
        self.covrad = LabeledCtrl(
            self,
            wx.ComboBox,
            'Covalent Radius (\xc5)',
            choices=[f'{p.covrad:.10g}' for p in ELEMENTS],
            style=wx.CB_READONLY,
            size=(1, -1),
        )
        self.vdwrad = LabeledCtrl(
            self,
            wx.ComboBox,
            'V.d.Waals Radius (\xc5)',
            choices=[f'{p.vdwrad:.10g}' for p in ELEMENTS],
            style=wx.CB_READONLY,
            size=(1, -1),
        )
        self.eleneg = LabeledCtrl(
            self,
            wx.ComboBox,
            'Electronegativity',
            choices=[f'{p.eleneg:.10g}' for p in ELEMENTS],
            style=wx.CB_READONLY,
            size=(1, -1),
        )
        self.eleconfig = LabeledCtrl(
            self,
            wx.ComboBox,
            'e- Config',
            choices=[p.eleconfig for p in ELEMENTS],
            style=wx.CB_READONLY,
            size=(1, -1),
        )
        self.eleshells = LabeledCtrl(
            self,
            wx.ComboBox,
            'Electrons per Shell',
            choices=[', '.join(f'{i}' for i in p.eleshells) for p in ELEMENTS],
            style=wx.CB_READONLY,
            size=(1, -1),
        )
        self.oxistates = LabeledCtrl(
            self,
            wx.ComboBox,
            'Oxidation States',
            choices=[p.oxistates for p in ELEMENTS],
            style=wx.CB_READONLY,
            size=(1, -1),
        )
        self.ionpot = LabeledCtrl(
            self,
            wx.Choice,
            'Ionization Potentials (eV)',
            choices=[],
            size=(1, -1),
        )
        self.isotopes = LabeledCtrl(
            self, wx.Choice, 'Isotopes', choices=[], size=(1, -1)
        )

        # layout
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer_top = wx.BoxSizer(wx.HORIZONTAL)
        sizer_left = wx.BoxSizer(wx.VERTICAL)
        sizer_right = wx.BoxSizer(wx.VERTICAL)
        sizer_num = wx.BoxSizer(wx.HORIZONTAL)
        style = wx.RIGHT | wx.BOTTOM | wx.EXPAND | wx.ADJUST_MINSIZE
        sizer_left.Add(self.names, 0, style, SPACER)
        sizer_left.Add(self.mass, 0, style, SPACER)
        sizer_left.Add(self.atmrad, 0, style, SPACER)
        sizer_left.Add(self.covrad, 0, style, SPACER)
        sizer_left.Add(self.vdwrad, 0, style, SPACER)
        sizer_left.Add(self.eleneg, 0, style, SPACER)
        sizer_top.Add(sizer_left, 1, wx.LEFT | wx.RIGHT, 0)
        style = wx.BOTTOM | wx.EXPAND | wx.ADJUST_MINSIZE
        sizer_num.Add(self.symbols, 1, style, 0)
        sizer_num.Add((SPACER, 5), 0, 0, 0)
        sizer_num.Add(self.numbers, 1, style, 0)
        sizer_right.Add(sizer_num, 0, style, SPACER)
        sizer_right.Add(self.eleconfig, 0, style, SPACER)
        sizer_right.Add(self.eleshells, 0, style, SPACER)
        sizer_right.Add(self.oxistates, 0, style, SPACER)
        sizer_right.Add(self.ionpot, 0, style, SPACER)
        sizer_right.Add(self.isotopes, 0, style, SPACER)
        sizer_top.Add(sizer_right, 1, wx.TOP | wx.RIGHT, 0)
        sizer.Add(
            sizer_top,
            1,
            wx.LEFT | wx.RIGHT | wx.TOP | wx.EXPAND | wx.ADJUST_MINSIZE,
            SPACER,
        )
        self.SetAutoLayout(True)
        self.SetSizerAndFit(sizer, True)
        sizer.SetSizeHints(self)
        self.Layout()

        # bind events
        self.Bind(wx.EVT_COMBOBOX, self.OnSelectName, self.names.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelectSymbol, self.symbols.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.numbers.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.mass.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.atmrad.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.covrad.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.vdwrad.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.eleshells.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.eleneg.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.eleconfig.ctrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnSelect, self.oxistates.ctrl)

    def SetSelection(self, select):
        """Set active element."""
        if self.selected == select:
            return
        self.selected = select
        ele = ELEMENTS[select + 1]

        self.names.ctrl.SetStringSelection(ele.name)
        self.symbols.ctrl.SetStringSelection(ele.symbol)
        self.numbers.ctrl.SetSelection(select)
        self.mass.ctrl.SetSelection(select)
        self.eleconfig.ctrl.SetSelection(select)
        self.atmrad.ctrl.SetSelection(select)
        self.covrad.ctrl.SetSelection(select)
        self.vdwrad.ctrl.SetSelection(select)
        self.eleneg.ctrl.SetSelection(select)
        self.eleshells.ctrl.SetSelection(select)
        self.oxistates.ctrl.SetSelection(select)

        self.isotopes.ctrl.Clear()
        for index, massnum in enumerate(sorted(ele.isotopes)):
            iso = ele.isotopes[massnum]
            self.isotopes.ctrl.Append(
                '{:3}:  {:8.4f} , {:8.4f}%'.format(
                    massnum, iso.mass, iso.abundance * 100.0
                )
            )
            if massnum == ele.nominalmass:
                self.isotopes.ctrl.SetSelection(index)

        self.ionpot.ctrl.Clear()
        for ion in ele.ionenergy:
            self.ionpot.ctrl.Append(f'{ion:8.4f}')
        self.ionpot.ctrl.SetSelection(0)

    def OnSelect(self, evt):
        self.SetSelection(evt.GetSelection())
        event = SelectionEvent(pteEVT_ELE_CHANGED, self.GetId(), self.selected)
        self.GetEventHandler().ProcessEvent(event)
        evt.Skip()

    def OnSelectName(self, evt):
        self.SetSelection(ELEMENTS[evt.GetString()].number - 1)
        event = SelectionEvent(pteEVT_ELE_CHANGED, self.GetId(), self.selected)
        self.GetEventHandler().ProcessEvent(event)
        evt.Skip()

    def OnSelectSymbol(self, evt):
        self.SetSelection(ELEMENTS[evt.GetString()].number - 1)
        event = SelectionEvent(pteEVT_ELE_CHANGED, self.GetId(), self.selected)
        self.GetEventHandler().ProcessEvent(event)
        evt.Skip()


class DecriptionPanel(wx.Panel):
    """Element description panel."""

    def __init__(self, *args, **kwds):
        kwds['style'] = wx.NO_BORDER | wx.TAB_TRAVERSAL
        wx.Panel.__init__(self, *args, **kwds)
        self.selected = -1

        self.description = wx.TextCtrl(
            self, -1, ' \n \n', style=wx.TE_MULTILINE | wx.TE_READONLY
        )
        font = self.description.GetFont()
        font.SetPointSize(font.GetPointSize() + 1)
        self.description.SetFont(font)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(
            self.description,
            1,
            wx.TOP
            | wx.LEFT
            | wx.RIGHT
            | wx.BOTTOM
            | wx.EXPAND
            | wx.FIXED_MINSIZE,
            SPACER,
        )

        self.SetAutoLayout(True)
        self.SetSizerAndFit(sizer, True)
        sizer.SetSizeHints(self)
        self.Layout()

    def SetSelection(self, select):
        """Set active element."""
        if self.selected == select:
            return
        self.selected = select

        ele = ELEMENTS[select + 1]
        self.description.SetValue(ele.description)


class ElementButton(buttons.GenToggleButton):
    """Button representing chemical element."""

    def __init__(self, *args, **kwds):
        buttons.GenToggleButton.__init__(self, *args, **kwds)
        self.color = wx.Colour(255, 255, 255)
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)

    def OnEraseBackground(self, event):
        pass

    def SetButtonColour(self, color):
        self.color = color

    def OnPaint(self, event):
        width, height = self.GetClientSize()
        dc = wx.BufferedPaintDC(self)
        brush = wx.Brush(self.GetBackgroundColour(), wx.BRUSHSTYLE_SOLID)
        if wx.Platform == '__WXMAC__':
            brush.MacSetTheme(1)  # kThemeBrushDialogBackgroundActive
        dc.SetBackground(brush)
        dc.SetPen(wx.TRANSPARENT_PEN)
        dc.SetBrush(
            wx.Brush(self.color if self.up else 'WHITE', wx.BRUSHSTYLE_SOLID)
        )
        dc.Clear()
        dc.DrawRectangle((1, 1), (width - 2, height - 2))
        if self.up:
            self.DrawBezel(dc, 0, 0, width - 1, height - 1)
        self.DrawLabel(dc, width, height)
        if self.hasFocus and self.useFocusInd:
            self.DrawFocusIndicator(dc, width, height)

    def DrawLabel(self, dc, width, height):
        font = self.GetFont()
        font.SetWeight(wx.FONTWEIGHT_BOLD)
        dc.SetFont(font)

        if self.IsEnabled():
            dc.SetTextForeground(self.GetForegroundColour())
        else:
            dc.SetTextForeground(
                wx.SystemSettings.GetColour(wx.SYS_COLOUR_GRAYTEXT)
            )

        label = self.GetLabel()
        txtwidth, txtheight = dc.GetTextExtent(label)
        xpos = (width - txtwidth) // 2
        ypos = ((height * 3) // 4 - txtheight) // 2 - 1
        dc.DrawText(label, xpos, ypos)

        font.SetWeight(wx.FONTWEIGHT_LIGHT)
        font.SetPointSize((font.GetPointSize() * 6) // 8)
        dc.SetFont(font)
        label = f'{self.GetId() - 100}'
        txtwidth, txtheight = dc.GetTextExtent(label)
        dc.DrawText(
            label,
            (width - txtwidth) // 2,
            4 + ypos + (height - txtheight) // 2,
        )


class LabeledCtrl(wx.BoxSizer):
    """BoxSizer containing label, control, and unit."""

    def __init__(self, parent, control, label, unit=None, space='  ', **kwds):
        wx.BoxSizer.__init__(self, wx.HORIZONTAL)
        self.label = wx.StaticText(parent, -1, label + space)
        self.ctrl = control(parent, -1, **kwds)
        self.Add(self.label, 0, wx.ALIGN_CENTER_VERTICAL | wx.FIXED_MINSIZE)
        self.Add(
            self.ctrl,
            1,
            wx.LEFT | wx.EXPAND | wx.ADJUST_MINSIZE,
            # | wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_RIGHT
            0,
        )
        if unit:
            self.unit = wx.StaticText(parent, -1, unit)
            self.Add(
                self.unit,
                0,
                wx.RIGHT | wx.ALIGN_CENTER_VERTICAL | wx.FIXED_MINSIZE,
                0,
            )
        else:
            self.unit = None


class StaticFancyText(fancytext.StaticFancyText):
    """StaticFancyText with SetLabel function."""

    def SetLabel(self, label):
        """Render label."""
        bmp = fancytext.RenderToBitmap(
            label, wx.Brush(self.GetBackgroundColour(), wx.BRUSHSTYLE_SOLID)
        )
        self.SetBitmap(bmp)


class DisclosureCtrl(buttons.GenBitmapTextToggleButton):
    """Disclosure triangle button."""

    bmp0 = (
        b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\r\x00\x00'
        b'\x00\r\x08\x06\x00\x00\x00r\xeb\xe4|\x00\x00\x00\x04sBIT'
        b'\x08\x08\x08\x08|\x08d\x88\x00\x00\x00\xd5IDAT(\x91\x9d'
        b'\x921n\x83@\x10E\xdf.!\x96\x9c\x90P"\xd9\x86\x92\x82\x03p'
        b'\x02h\xa9i9A\xe2\xb38)9\x02w\xc0\t\x94I\xa0H\xd2\xc7'
        b'\x96(\x918\xc0\xa6Z\x17\x91\x05&_\x1a\xe9k4O\xfa3\x1a!'
        b'\xa4\xc1\\Im>\xde\xdf\xd4l\xa8,K\x9e\x9fv\xeax\xf8\x99\x84'
        b'\x85\x8e\xb7}|P\x00\xb6m\x13\x04\x01q\x1cc\xdd\xdd\x8bs\xd0'
        b'\xd5\xdfF\xdf\xf7TUE\xdb\xb6\xbc\xbe\xecU\x18\x86\x98\xd7'
        b'\x0b1\ni\r\xc3@Q\x14\xd4u\xcd\xf7\xd7\xa7r]\x97\xe5\xcd'
        b'\xad\x18\x85\xb4\xba\xae#\xcfs|\xdf?\xf5\xe4\xc8<\x00\x8e'
        b'\xe3\x90e\x19i\x9aN\xc7\xb3,\x8b(\x8a\xb8h\xa7Y\xd7'
        b'\xf3<\x0f\xd34I\x92\x84\xd5zsv\xf8$!\r\x844h\x9aFi?Y\xff'
        b'\xf9\xbd_\xd7\x8c7Z\xc0k\x8d8\x00\x00\x00\x00IEND\xaeB`\x82'
    )

    bmp1 = (
        b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\r\x00\x00'
        b'\x00\r\x08\x06\x00\x00\x00r\xeb\xe4|\x00\x00\x00\x04sBIT'
        b'\x08\x08\x08\x08|\x08d\x88\x00\x00\x00\xc1IDAT(\x91\x9d\xd2;'
        b'\x8e\xc20\x10\x80\xe1\x7fB\x0c\x95\xc1\xa5%\xc0)S\xb8I\xd2'
        b'\xb8Le\x97\xf6\x99s\x81\xdd\xbdBN\xb2\xdb!Y\t\xac`\xbay|'
        b'\xa3)F\xa49\xf0n4o\x8bOQ\x0b\xf0\xf3\xfd\xf5\xbb,\x0b\xeb'
        b'\xba>\x1d\xec\xba\x8ey\x9e\x19\xc6I\x1a\x80a\x9cD)\xf5r'
        b'\xbbR\x8aa\x9c\xa4:\xaf\x94\x821f\x17\x18c(\xa5<\xf2\x07'
        b'\xba\xde\xee\xe2\xbd\xdfE\xde{\xae\xb7\xbbl\x10@J\t\xadu\x05'
        b'\xb4\xd6\xa4\x94\xaaZ\x85\xf4\xf9"1\xc6j \xc6\x88>_\xe4)\x02'
        b'\x08!`\xad\x05\xc0ZK\x08as\xee\x06\xa9\xe3Ir\xce\xb4mK\xce'
        b'\x19u<\xc9\xbf\x08\xc09G\xdf\xf78\xe7\xf6\xda\xc8\'\xbf\xf7'
        b'\x07\x13\x12\x18B\x17\x9fx\xa0\x00\x00\x00\x00IEND\xaeB`\x82'
    )

    def __init__(self, parent, winid, label, *args, **kwds):
        kwds['style'] = wx.BORDER_NONE | wx.BU_EXACTFIT
        buttons.GenBitmapTextToggleButton.__init__(
            self, parent, winid, None, label, *args, **kwds
        )

        self.bmp0 = wx.Bitmap(wx.Image(io.BytesIO(self.bmp0)))
        self.bmp1 = wx.Bitmap(wx.Image(io.BytesIO(self.bmp1)))

        self.SetBitmapLabel(self.bmp0)
        self.SetBitmapSelected(self.bmp1)
        if not label:
            self.SetSize(self.bmp0.Size)
        else:
            self.SetBestSize()
        self.labelDelta = 0
        self.useFocusInd = False
        self.SetToolTip('Show')
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)

    def OnEraseBackground(self, event):
        pass

    def Notify(self):
        wx.lib.buttons.GenBitmapTextToggleButton.Notify(self)
        self.SetToolTip('Show' if self.up else 'Hide')

    def DoGetBestSize(self):
        width, height, usemin = self._GetLabelSize()
        return (width + 5, height + 4)

    def OnPaint(self, event):
        width, height = self.GetClientSize()
        dc = wx.BufferedPaintDC(self)
        brush = None
        bgcol = self.GetBackgroundColour()
        brush = wx.Brush(bgcol, wx.BRUSHSTYLE_SOLID)
        defattr = self.GetDefaultAttributes()
        if self.style & wx.BORDER_NONE and bgcol == defattr.colBg:
            defattr = self.GetParent().GetDefaultAttributes()
            # if self.GetParent().GetBackgroundColour() == defattr.colBg:
            #     if wx.Platform == '__WXMSW__':
            #         if (
            #             hasattr(self, 'DoEraseBackground') and
            #             self.DoEraseBackground(dc)
            #         ):
            #             brush = None
            #     elif wx.Platform == '__WXMAC__':
            #         brush.MacSetTheme(1)
            # else:
            #     bgcol = self.GetParent().GetBackgroundColour()
            #     brush = wx.Brush(bgcol, wx.BRUSHSTYLE_SOLID)
        if brush is not None:
            dc.SetBackground(brush)
            dc.Clear()
        self.DrawLabel(dc, width, height)

    def DrawLabel(self, dc, width, height):
        bmp = self.bmpLabel
        if bmp is not None:
            if self.bmpDisabled and not self.IsEnabled():
                bmp = self.bmpDisabled
            if self.bmpFocus and self.hasFocus:
                bmp = self.bmpFocus
            if self.bmpSelected and not self.up:
                bmp = self.bmpSelected
            bmpwidth, bmpheight = bmp.GetWidth(), bmp.GetHeight()
            hasmask = bmp.GetMask() is not None
        else:
            bmpwidth = bmpheight = 0

        dc.SetFont(self.GetFont())
        color = (
            self.GetForegroundColour()
            if self.IsEnabled()
            else wx.SystemSettings.GetColour(wx.SYS_COLOUR_GRAYTEXT)
        )
        dc.SetTextForeground(color)

        label = self.GetLabel()
        txtwidth, txtheight = dc.GetTextExtent(label)
        # center bitmap and text
        # xpos = (width - bmpwidth - txtwidth) // 2 if center else 0
        xpos = 0
        if bmp is not None:
            dc.DrawBitmap(bmp, xpos, (height - bmpheight) // 2, hasmask)
            xpos += 5
        dc.DrawText(label, xpos + bmpwidth, (height - txtheight) // 2)


class SelectionEvent(wx.PyCommandEvent):
    """Notification of changed element."""

    def __init__(self, evtType, winid, sel):
        wx.PyCommandEvent.__init__(self, evtType, winid)
        self.selection = sel

    def SetSelection(self, select):
        self.selection = select

    def GetSelection(self):
        return self.selection


pteEVT_ELE_CHANGED = wx.NewEventType()
EVT_ELE_CHANGED = wx.PyEventBinder(pteEVT_ELE_CHANGED, 1)

SPACER = 12 if wx.Platform == '__WXMAC__' else 10
BORDER = 22 if wx.Platform == '__WXMAC__' else 10

COLORS = {
    1: (0x99, 0xFF, 0x99),  # Nonmetals
    2: (0xC0, 0xFF, 0xFF),  # Noble gases
    3: (0xFF, 0x99, 0x99),  # Alkali metals
    4: (0xFF, 0xDE, 0xAD),  # Alkaline earth metals
    5: (0xCC, 0xCC, 0x99),  # Metalloids
    6: (0xFF, 0xFF, 0x99),  # Halogens
    7: (0xCC, 0xCC, 0xCC),  # Poor metals
    8: (0xFF, 0xC0, 0xC0),  # Transition metals
    9: (0xFF, 0xBF, 0xFF),  # Lanthanides
    10: (0xFF, 0x99, 0xCC),  # Actinides
}


def main():
    """Run the application."""
    MainApp(0).MainLoop()


if __name__ == '__main__':
    main()



# elements_descriptions.py

"""Descriptions of the chemical elements."""


def elements_descriptions(elements, symbol=None):
    """Set the description attribute on all elements."""
    elements['H'].description = (
        "Colourless, odourless gaseous chemical element. Lightest and most "
        "abundant element in the universe. Present in water and in all "
        "organic compounds. Chemically reacts with most elements. Discovered "
        "by Henry Cavendish in 1776."
    )
    elements['He'].description = (
        "Colourless, odourless gaseous nonmetallic element. Belongs to "
        "group 18 of the periodic table. Lowest boiling point of all "
        "elements and can only be solidified under pressure. Chemically "
        "inert, no known compounds. Discovered in the solar spectrum in 1868 "
        "by Lockyer."
    )
    elements['Li'].description = (
        "Socket silvery metal. First member of group 1 of the periodic "
        "table. Lithium salts are used in psychomedicine."
    )
    elements['Be'].description = (
        "Grey metallic element of group 2 of the periodic table. Is toxic "
        "and can cause severe lung diseases and dermatitis. Shows high "
        "covalent character. It was isolated independently by F. Wohler and "
        "A.A. Bussy in 1828."
    )
    elements['B'].description = (
        "An element of group 13 of the periodic table. There are two "
        "allotropes, amorphous boron is a brown power, but metallic boron is "
        "black. The metallic form is hard (9.3 on Mohs' scale) and a bad "
        "conductor in room temperatures. It is never found free in nature. "
        "Boron-10 is used in nuclear reactor control rods and shields. It "
        "was discovered in 1808 by Sir Humphry Davy and by J.L. Gay-Lussac "
        "and L.J. Thenard."
    )
    elements['C'].description = (
        "Carbon is a member of group 14 of the periodic table. It has three "
        "allotropic forms of it, diamonds, graphite and fullerite. Carbon-14 "
        "is commonly used in radioactive dating. Carbon occurs in all "
        "organic life and is the basis of organic chemistry. Carbon has the "
        "interesting chemical property of being able to bond with itself, "
        "and a wide variety of other elements."
    )
    elements['N'].description = (
        "Colourless, gaseous element which belongs to group 15 of the "
        "periodic table. Constitutes ~78% of the atmosphere and is an "
        "essential part of the ecosystem. Nitrogen for industrial purposes "
        "is acquired by the fractional distillation of liquid air. "
        "Chemically inactive, reactive generally only at high temperatures "
        "or in electrical discharges. It was discovered in 1772 by D. "
        "Rutherford."
    )
    elements['O'].description = (
        "A colourless, odourless gaseous element belonging to group 16 of "
        "the periodic table. It is the most abundant element present in the "
        "earth's crust. It also makes up 20.8% of the Earth's atmosphere. "
        "For industrial purposes, it is separated from liquid air by "
        "fractional distillation. It is used in high temperature welding, "
        "and in breathing. It commonly comes in the form of Oxygen, but is "
        "found as Ozone in the upper atmosphere. It was discovered by "
        "Priestley in 1774."
    )
    elements['F'].description = (
        "A poisonous pale yellow gaseous element belonging to group 17 of "
        "the periodic table (The halogens). It is the most chemically "
        "reactive and electronegative element. It is highly dangerous, "
        "causing severe chemical burns on contact with flesh. Fluorine was "
        "identified by Scheele in 1771 and first isolated by Moissan in "
        "1886."
    )
    elements['Ne'].description = (
        "Colourless gaseous element of group 18 on the periodic table "
        "(noble gases). Neon occurs in the atmosphere, and comprises 0.0018% "
        "of the volume of the atmosphere. It has a distinct reddish glow "
        "when used in discharge tubes and neon based lamps. It forms almost "
        "no chemical compounds. Neon was discovered in 1898 by Sir William "
        "Ramsey and M.W. Travers."
    )
    elements['Na'].description = (
        "Soft silvery reactive element belonging to group 1 of the periodic "
        "table (alkali metals). It is highly reactive, oxidizing in air and "
        "reacting violently with water, forcing it to be kept under oil. It "
        "was first isolated by Humphrey Davy in 1807."
    )
    elements['Mg'].description = (
        "Silvery metallic element belonging to group 2 of the periodic "
        "table (alkaline-earth metals). It is essential for living "
        "organisms, and is used in a number of light alloys. Chemically very "
        "reactive, it forms a protective oxide coating when exposed to air "
        "and burns with an intense white flame. It also reacts with sulphur, "
        "nitrogen and the halogens. First isolated by Bussy in 1828."
    )
    elements['Al'].description = (
        "Silvery-white lustrous metallic element of group 3 of the periodic "
        "table. Highly reactive but protected by a thin transparent layer of "
        "the oxide which quickly forms in air. There are many alloys of "
        "aluminum, as well as a good number of industrial uses. Makes up "
        "8.1% of the Earth's crust, by weight. Isolated in 1825 by H.C. "
        "Oersted."
    )
    elements['Si'].description = (
        "Metalloid element belonging to group 14 of the periodic table. It "
        "is the second most abundant element in the Earth's crust, making up "
        "25.7% of it by weight. Chemically less reactive than carbon. First "
        "identified by Lavoisier in 1787 and first isolated in 1823 by "
        "Berzelius."
    )
    elements['P'].description = (
        "Non-metallic element belonging to group 15 of the periodic table. "
        "Has a multiple allotropic forms. Essential element for living "
        "organisms. It was discovered by Brandt in 1669."
    )
    elements['S'].description = (
        "Yellow, nonmetallic element belonging to group 16 of the periodic "
        "table. It is an essential element in living organisms, needed in "
        "the amino acids cysteine and methionine, and hence in many "
        "proteins. Absorbed by plants from the soil as sulphate ion."
    )
    elements['Cl'].description = (
        "Halogen element. Poisonous greenish-yellow gas. Occurs widely in "
        "nature as sodium chloride in seawater. Reacts directly with many "
        "elements and compounds, strong oxidizing agent. Discovered by Karl "
        "Scheele in 1774. Humphrey David confirmed it as an element in 1810."
    )
    elements['Ar'].description = (
        "Monatomic noble gas. Makes up 0.93% of the air. Colourless, "
        "odorless. Is inert and has no true compounds. Lord Rayleigh and Sir "
        "william Ramsey identified argon in 1894."
    )
    elements['K'].description = (
        "Soft silvery metallic element belonging to group 1 of the periodic "
        "table (alkali metals). Occurs naturally in seawater and a many "
        "minerals. Highly reactive, chemically, it resembles sodium in its "
        "behavior and compounds. Discovered by Sir Humphry Davy in 1807."
    )
    elements['Ca'].description = (
        "Soft grey metallic element belonging to group 2 of the periodic "
        "table. Used a reducing agent in the extraction of thorium, "
        "zirconium and uranium. Essential element for living organisms."
    )
    elements['Sc'].description = (
        "Rare soft silvery metallic element belonging to group 3 of the "
        "periodic table. There are ten isotopes, nine of which are "
        "radioactive and have short half-lives. Predicted in 1869 by "
        "Mendeleev, isolated by Nilson in 1879."
    )
    elements['Ti'].description = (
        "White metallic transition element. Occurs in numerous minerals. "
        "Used in strong, light corrosion-resistant alloys. Forms a passive "
        "oxide coating when exposed to air. First discovered by Gregor in "
        "1789."
    )
    elements['V'].description = (
        "Soft and ductile, bright white metal. Good resistance to corrosion "
        "by alkalis, sulphuric and hydrochloric acid. It oxidizes readily "
        "about 933K. There are two naturally occurring isotopes of vanadium, "
        "and 5 radioisotopes, V-49 having the longest half-life at 337 days. "
        "Vanadium has nuclear applications, the foil is used in cladding "
        "titanium to steel, and vanadium-gallium tape is used to produce a "
        "superconductive magnet. Originally discovered by Andres Manuel del "
        "Rio of Mexico City in 1801. His discovery went unheeded, however, "
        "and in 1820, Nils Gabriel Sefstron of Sweden rediscovered it. "
        "Metallic vanadium was isolated by Henry Enfield Roscoe in 1867. The "
        "name vanadium comes from Vanadis, a goddess of Scandinavian "
        "mythology. Silvery-white metallic transition element. Vanadium is "
        "essential to Ascidians. Rats and chickens are also known to require "
        "it. Metal powder is a fire hazard, and vanadium compounds should be "
        "considered highly toxic. May cause lung cancer if inhaled."
    )
    elements['Cr'].description = (
        "Hard silvery transition element. Used in decorative "
        "electroplating. Discovered in 1797 by Vauquelin."
    )
    elements['Mn'].description = (
        "Grey brittle metallic transition element. Rather electropositive, "
        "combines with some non-metals when heated. Discovered in 1774 by "
        "Scheele."
    )
    elements['Fe'].description = (
        "Silvery malleable and ductile metallic transition element. Has "
        "nine isotopes and is the fourth most abundant element in the "
        "earth's crust. Required by living organisms as a trace element "
        "(used in hemoglobin in humans.) Quite reactive, oxidizes in moist "
        "air, displaces hydrogen from dilute acids and combines with "
        "nonmetallic elements."
    )
    elements['Co'].description = (
        "Light grey transition element. Some meteorites contain small "
        "amounts of metallic cobalt. Generally alloyed for use. Mammals "
        "require small amounts of cobalt salts. Cobalt-60, an artificially "
        "produced radioactive isotope of Cobalt is an important radioactive "
        "tracer and cancer-treatment agent. Discovered by G. Brandt in 1737."
    )
    elements['Ni'].description = (
        "Malleable ductile silvery metallic transition element. Discovered "
        "by A.F. Cronstedt in 1751."
    )
    elements['Cu'].description = (
        "Red-brown transition element. Known by the Romans as 'cuprum.' "
        "Extracted and used for thousands of years. Malleable, ductile and "
        "an excellent conductor of heat and electricity. When in moist "
        "conditions, a greenish layer forms on the outside."
    )
    elements['Zn'].description = (
        "Blue-white metallic element. Occurs in multiple compounds "
        "naturally. Five stable isotopes are six radioactive isotopes have "
        "been found. Chemically a reactive metal, combines with oxygen and "
        "other non-metals, reacts with dilute acids to release hydrogen."
    )
    elements['Ga'].description = (
        "Soft silvery metallic element, belongs to group 13 of the periodic "
        "table. The two stable isotopes are Ga-69 and Ga-71. Eight "
        "radioactive isotopes are known, all having short half-lives. "
        "Gallium Arsenide is used as a semiconductor. Corrodes most other "
        "metals by diffusing into their lattice. First identified by "
        "Francois Lecoq de Boisbaudran in 1875."
    )
    elements['Ge'].description = (
        "Lustrous hard metalloid element, belongs to group 14 of the "
        "periodic table. Forms a large number of organometallic compounds. "
        "Predicted by Mendeleev in 1871, it was actually found in 1886 by "
        "Winkler."
    )
    elements['As'].description = (
        "Metalloid element of group 15. There are three allotropes, yellow, "
        "black, and grey. Reacts with halogens, concentrated oxidizing acids "
        "and hot alkalis. Albertus Magnus is believed to have been the first "
        "to isolate the element in 1250."
    )
    elements['Se'].description = (
        "Metalloid element, belongs to group 16 of the periodic table. "
        "Multiple allotropic forms exist. Chemically resembles sulphur. "
        "Discovered in 1817 by Jons J. Berzelius."
    )
    elements['Br'].description = (
        "Halogen element. Red volatile liquid at room temperature. Its "
        "reactivity is somewhere between chlorine and iodine. Harmful to "
        "human tissue in a liquid state, the vapour irritates eyes and "
        "throat. Discovered in 1826 by Antoine Balard."
    )
    elements['Kr'].description = (
        "Colorless gaseous element, belongs to the noble gases. Occurs in "
        "the air, 0.0001% by volume. It can be extracted from liquid air by "
        "fractional distillation. Generally not isolated, but used with "
        "other inert gases in fluorescent lamps. Five natural isotopes, and "
        "five radioactive isotopes. Kr-85, the most stable radioactive "
        "isotope, has a half-life of 10.76 years and is produced in fission "
        "reactors. Practically inert, though known to form compounds with "
        "Fluorine."
    )
    elements['Rb'].description = (
        "Soft silvery metallic element, belongs to group 1 of the periodic "
        "table. Rb-97, the naturally occurring isotope, is radioactive. It "
        "is highly reactive, with properties similar to other elements in "
        "group 1, like igniting spontaneously in air. Discovered "
        "spectroscopically in 1861 by W. Bunsen and G.R. Kirchoff."
    )
    elements['Sr'].description = (
        "Soft yellowish metallic element, belongs to group 2 of the "
        "periodic table. Highly reactive chemically. Sr-90 is present in "
        "radioactive fallout and has a half-life of 28 years. Discovered in "
        "1798 by Klaproth and Hope, isolated in 1808 by Humphry Davy."
    )
    elements['Y'].description = (
        "Silvery-grey metallic element of group 3 on the periodic table. "
        "Found in uranium ores. The only natural isotope is Y-89, there are "
        "14 other artificial isotopes. Chemically resembles the lanthanoids. "
        "Stable in the air below 400 degrees, celsius. Discovered in 1828 by "
        "Friedrich Wohler."
    )
    elements['Zr'].description = (
        "Grey-white metallic transition element. Five natural isotopes and "
        "six radioactive isotopes are known. Used in nuclear reactors for a "
        "Neutron absorber. Discovered in 1789 by Martin Klaproth, isolated "
        "in 1824 by Berzelius."
    )
    elements['Nb'].description = (
        "Soft, ductile grey-blue metallic transition element. Used in "
        "special steels and in welded joints to increase strength. Combines "
        "with halogens and oxidizes in air at 200 degrees celsius. "
        "Discovered by Charles Hatchett in 1801 and isolated by Blomstrand "
        "in 1864. Called Columbium originally."
    )
    elements['Mo'].description = (
        "Silvery-white, hard metallic transition element. It is chemically "
        "unreactive and is not affected by most acids. It oxidizes at high "
        "temperatures. There are seven natural isotopes, and four "
        "radioisotopes, Mo-93 being the most stable with a half-life of 3500 "
        "years. Molybdenum is used in almost all high-strength steels, it "
        "has nuclear applications, and is a catalyst in petroleum refining. "
        "Discovered in 1778 by Carl Welhelm Scheele of Sweden. Impure metal "
        "was prepared in 1782 by Peter Jacob Hjelm. The name comes from the "
        "Greek word molybdos which means lead. Trace amounts of molybdenum "
        "are required for all known forms of life. All molybdenum compounds "
        "should be considered highly toxic, and will also cause severe birth "
        "defects."
    )
    elements['Tc'].description = (
        "Radioactive metallic transition element. Can be detected in some "
        "stars and the fission products of uranium. First made by Perrier "
        "and Segre by bombarding molybdenum with deutrons, giving them "
        "Tc-97. Tc-99 is the most stable isotope with a half-life of "
        "2.6*10^6 years. Sixteen isotopes are known. Organic technetium "
        "compounds are used in bone imaging. Chemical properties are "
        "intermediate between rhenium and manganese."
    )
    elements['Ru'].description = (
        "Hard white metallic transition element. Found with platinum, used "
        "as a catalyst in some platinum alloys. Dissolves in fused alkalis, "
        "and is not attacked by acids. Reacts with halogens and oxygen at "
        "high temperatures. Isolated in 1844 by K.K. Klaus."
    )
    elements['Rh'].description = (
        "Silvery white metallic transition element. Found with platinum and "
        "used in some platinum alloys. Not attacked by acids, dissolves only "
        "in aqua regia. Discovered in 1803 by W.H. Wollaston."
    )
    elements['Pd'].description = (
        "Soft white ductile transition element. Found with some copper and "
        "nickel ores. Does not react with oxygen at normal temperatures. "
        "Dissolves slowly in hydrochloric acid. Discovered in 1803 by W.H. "
        "Wollaston."
    )
    elements['Ag'].description = (
        "White lustrous soft metallic transition element. Found in both its "
        "elemental form and in minerals. Used in jewellery, tableware and so "
        "on. Less reactive than silver, chemically."
    )
    elements['Cd'].description = (
        "Soft bluish metal belonging to group 12 of the periodic table. "
        "Extremely toxic even in low concentrations. Chemically similar to "
        "zinc, but lends itself to more complex compounds. Discovered in "
        "1817 by F. Stromeyer."
    )
    elements['In'].description = (
        "Soft silvery element belonging to group 13 of the periodic table. "
        "The most common natural isotope is In-115, which has a half-life of "
        "6*10^4 years. Five other radioisotopes exist. Discovered in 1863 by "
        "Reich and Richter."
    )
    elements['Sn'].description = (
        "Silvery malleable metallic element belonging to group 14 of the "
        "periodic table. Twenty-six isotopes are known, five of which are "
        "radioactive. Chemically reactive. Combines directly with chlorine "
        "and oxygen and displaces hydrogen from dilute acids."
    )
    elements['Sb'].description = (
        "Element of group 15. Multiple allotropic forms. The stable form of "
        "antimony is a blue-white metal. Yellow and black antimony are "
        "unstable non-metals. Used in flame-proofing, paints, ceramics, "
        "enamels, and rubber. Attacked by oxidizing acids and halogens. "
        "First reported by Tholden in 1450."
    )
    elements['Te'].description = (
        "Silvery metalloid element of group 16. Eight natural isotopes, "
        "nine radioactive isotopes. Used in semiconductors and to a degree "
        "in some steels. Chemistry is similar to Sulphur. Discovered in 1782 "
        "by Franz Miller."
    )
    elements['I'].description = (
        "Dark violet nonmetallic element, belongs to group 17 of the "
        "periodic table. Insoluble in water. Required as a trace element for "
        "living organisms. One stable isotope, I-127 exists, in addition to "
        "fourteen radioactive isotopes. Chemically the least reactive of the "
        "halogens, and the most electropositive metallic halogen. Discovered "
        "in 1812 by Courtois."
    )
    elements['Xe'].description = (
        "Colourless, odourless gas belonging to group 18 on the periodic "
        "table (the noble gases.) Nine natural isotopes and seven "
        "radioactive isotopes are known. Xenon was part of the first "
        "noble-gas compound synthesized. Several others involving Xenon have "
        "been found since then. Xenon was discovered by Ramsey and Travers "
        "in 1898."
    )
    elements['Cs'].description = (
        "Soft silvery-white metallic element belonging to group 1 of the "
        "periodic table. One of the three metals which are liquid at room "
        "temperature. Cs-133 is the natural, and only stable, isotope. "
        "Fifteen other radioisotopes exist. Caesium reacts explosively with "
        "cold water, and ice at temperatures above 157K. Caesium hydroxide "
        "is the strongest base known. Caesium is the most electropositive, "
        "most alkaline and has the least ionization potential of all the "
        "elements. Known uses include the basis of atomic clocks, catalyst "
        "for the hydrogenation of some organic compounds, and in "
        "photoelectric cells. Caesium was discovered by Gustav Kirchoff and "
        "Robert Bunsen in Germany in 1860 spectroscopically. Its "
        "identification was based upon the bright blue lines in its "
        "spectrum. The name comes from the latin word caesius, which means "
        "sky blue. Caesium should be considered highly toxic. Some of the "
        "radioisotopes are even more toxic."
    )
    elements['Ba'].description = (
        "Silvery-white reactive element, belonging to group 2 of the "
        "periodic table. Soluble barium compounds are extremely poisonous. "
        "Identified in 1774 by Karl Scheele and extracted in 1808 by Humphry "
        "Davy."
    )
    elements['La'].description = (
        "(From the Greek word lanthanein, to line hidden) Silvery metallic "
        "element belonging to group 3 of the periodic table and oft "
        "considered to be one of the lanthanoids. Found in some rare-earth "
        "minerals. Twenty-five natural isotopes exist. La-139 which is "
        "stable, and La-138 which has a half-life of 10^10 to 10^15 years. "
        "The other twenty-three isotopes are radioactive. It resembles the "
        "lanthanoids chemically. Lanthanum has a low to moderate level of "
        "toxicity, and should be handled with care. Discovered in 1839 by "
        "C.G. Mosander."
    )
    elements['Ce'].description = (
        "Silvery metallic element, belongs to the lanthanoids. Four natural "
        "isotopes exist, and fifteen radioactive isotopes have been "
        "identified. Used in some rare-earth alloys. The oxidized form is "
        "used in the glass industry. Discovered by Martin .H. Klaproth in "
        "1803."
    )
    elements['Pr'].description = (
        "Soft silvery metallic element, belongs to the lanthanoids. Only "
        "natural isotope is Pr-141 which is not radioactive. Fourteen "
        "radioactive isotopes have been artificially produced. Used in "
        "rare-earth alloys. Discovered in 1885 by C.A. von Welsbach."
    )
    elements['Nd'].description = (
        "Soft bright silvery metallic element, belongs to the lanthanoids. "
        "Seven natural isotopes, Nd-144 being the only radioactive one with "
        "a half-life of 10^10 to 10^15 years. Six artificial radioisotopes "
        "have been produced. The metal is used in glass works to color class "
        "a shade of violet-purple and make it dichroic. One of the more "
        "reactive rare-earth metals, quickly reacts with air. Used in some "
        "rare-earth alloys. Neodymium is used to color the glass used in "
        "welder's glasses. Neodymium is also used in very powerful, "
        "permanent magnets (Nd2Fe14B). Discovered by Carl F. Auer von "
        "Welsbach in Austria in 1885 by separating didymium into its "
        "elemental components Praseodymium and neodymium. The name comes "
        "from the Greek words 'neos didymos' which means 'new twin'. "
        "Neodymium should be considered highly toxic, however evidence would "
        "seem to show that it acts as little more than a skin and eye "
        "irritant. The dust however, presents a fire and explosion hazard."
    )
    elements['Pm'].description = (
        "Soft silvery metallic element, belongs to the lanthanoids. Pm-147, "
        "the only natural isotope, is radioactive and has a half-life of 252 "
        "years. Eighteen radioisotopes have been produced, but all have very "
        "short half-lives. Found only in nuclear decay waste. Pm-147 is of "
        "interest as a beta-decay source, however Pm-146 and Pm-148 have to "
        "be removed from it first, as they generate gamma radiation. "
        "Discovered by J.A. Marinsky, L.E. Glendenin and C.D. Coryell in "
        "1947."
    )
    elements['Sm'].description = (
        "Soft silvery metallic element, belongs to the lanthanoids. Seven "
        "natural isotopes, Sm-147 is the only radioisotope, and has a "
        "half-life of 2.5*10^11 years. Used for making special alloys needed "
        "in the production of nuclear reactors. Also used as a neutron "
        "absorber. Small quantities of samarium oxide is used in special "
        "optical glasses. The largest use of the element is its "
        "ferromagnetic alloy which produces permanent magnets that are five "
        "times stronger than magnets produced by any other material. "
        "Discovered by Francois Lecoq de Boisbaudran in 1879."
    )
    elements['Eu'].description = (
        "Soft silvery metallic element belonging to the lanthanoids. Eu-151 "
        "and Eu-153 are the only two stable isotopes, both of which are "
        "Neutron absorbers. Discovered in 1889 by Sir William Crookes."
    )
    elements['Gd'].description = (
        "Soft silvery metallic element belonging to the lanthanoids. Seven "
        "natural, stable isotopes are known in addition to eleven artificial "
        "isotopes. Gd-155 and Gd-157 and the best neutron absorbers of all "
        "elements. Gadolinium compounds are used in electronics. Discovered "
        "by J.C.G Marignac in 1880."
    )
    elements['Tb'].description = (
        "Silvery metallic element belonging to the lanthanoids. Tb-159 is "
        "the only stable isotope, there are seventeen artificial isotopes. "
        "Discovered by G.G. Mosander in 1843."
    )
    elements['Dy'].description = (
        "Metallic with a bright silvery-white lustre. Dysprosium belongs to "
        "the lanthanoids. It is relatively stable in air at room "
        "temperatures, it will however dissolve in mineral acids, evolving "
        "hydrogen. It is found in from rare-earth minerals. There are seven "
        "natural isotopes of dysprosium, and eight radioisotopes, Dy-154 "
        "being the most stable with a half-life of 3*10^6 years. Dysprosium "
        "is used as a neutron absorber in nuclear fission reactions, and in "
        "compact disks. It was discovered by Paul Emile Lecoq de Boisbaudran "
        "in 1886 in France. Its name comes from the Greek word dysprositos, "
        "which means hard to obtain."
    )
    elements['Ho'].description = (
        "Relatively soft and malleable silvery-white metallic element, "
        "which is stable in dry air at room temperature. It oxidizes in "
        "moist air and at high temperatures. It belongs to the lanthanoids. "
        "A rare-earth metal, it is found in the minerals monazite and "
        "gadolinite. It possesses unusual magnetic properties. One natural "
        "isotope, Ho-165 exists, six radioisotopes exist, the most stable "
        "being Ho-163 with a half-life of 4570 years. Holmium is used in "
        "some metal alloys, it is also said to stimulate the metabolism. "
        "Discovered by Per Theodor Cleve and J.L. Soret in Switzerland in "
        "1879. The name homium comes from the Greek word Holmia which means "
        "Sweden. While all holmium compounds should be considered highly "
        "toxic, initial evidence seems to indicate that they do not pose "
        "much danger. The metal's dust however, is a fire hazard."
    )
    elements['Er'].description = (
        "Soft silvery metallic element which belongs to the lanthanoids. "
        "Six natural isotopes that are stable. Twelve artificial isotopes "
        "are known. Used in nuclear technology as a neutron absorber. It is "
        "being investigated for other possible uses. Discovered by Carl G. "
        "Mosander in 1843."
    )
    elements['Tm'].description = (
        "Soft grey metallic element that belongs to the lanthanoids. One "
        "natural isotope exists, Tm-169, and seventeen artificial isotopes "
        "have been produced. No known uses for the element. Discovered in "
        "1879 by Per Theodor Cleve."
    )
    elements['Yb'].description = (
        "Silvery metallic element of the lanthanoids. Seven natural "
        "isotopes and ten artificial isotopes are known. Used in certain "
        "steels. Discovered by J.D.G. Marignac in 1878."
    )
    elements['Lu'].description = (
        "Silvery-white rare-earth metal which is relatively stable in air. "
        "It happens to be the most expensive rare-earth metal. Its found "
        "with almost all rare-earth metals, but is very difficult to "
        "separate from other elements. Least abundant of all natural "
        "elements. Used in metal alloys, and as a catalyst in various "
        "processes. There are two natural, stable isotopes, and seven "
        "radioisotopes, the most stable being Lu-174 with a half-life of 3.3 "
        "years. The separation of lutetium from Ytterbium was described by "
        "Georges Urbain in 1907. It was discovered at approximately the same "
        "time by Carl Auer von Welsbach. The name comes from the Greek word "
        "lutetia which means Paris."
    )
    elements['Hf'].description = (
        "Silvery lustrous metallic transition element. Used in tungsten "
        "alloys in filaments and electrodes, also acts as a neutron "
        "absorber. First reported by Urbain in 1911, existence was finally "
        "established in 1923 by D. Coster, G.C. de Hevesy in 1923."
    )
    elements['Ta'].description = (
        "Heavy blue-grey metallic transition element. Ta-181 is a stable "
        "isotope, and Ta-180 is a radioactive isotope, with a half-life in "
        "excess of 10^7 years. Used in surgery as it is unreactive. Forms a "
        "passive oxide layer in air. Identified in 1802 by Ekeberg and "
        "isolated in 1820 by Jons J. Berzelius."
    )
    elements['W'].description = (
        "White or grey metallic transition element,formerly called Wolfram. "
        "Forms a protective oxide in air and can be oxidized at high "
        "temperature. First isolated by Jose and Fausto de Elhuyer in 1783."
    )
    elements['Re'].description = (
        "Silvery-white metallic transition element. Obtained as a "
        "by-product of molybdenum refinement. Rhenium-molybdenum alloys are "
        "superconducting."
    )
    elements['Os'].description = (
        "Hard blue-white metallic transition element. Found with platinum "
        "and used in some alloys with platinum and iridium."
    )
    elements['Ir'].description = (
        "Very hard and brittle, silvery metallic transition element. It has "
        "a yellowish cast to it. Salts of iridium are highly colored. It is "
        "the most corrosion resistant metal known, not attacked by any acid, "
        "but is attacked by molten salts. There are two natural isotopes of "
        "iridium, and 4 radioisotopes, the most stable being Ir-192 with a "
        "half-life of 73.83 days. Ir-192 decays into Platinum, while the "
        "other radioisotopes decay into Osmium. Iridium is used in high "
        "temperature apparatus, electrical contacts, and as a hardening "
        "agent for platinumpy. Discovered in 1803 by Smithson Tennant in "
        "England. The name comes from the Greek word iris, which means "
        "rainbow. Iridium metal is generally non-toxic due to its relative "
        "unreactivity, but iridium compounds should be considered highly "
        "toxic."
    )
    elements['Pt'].description = (
        "Attractive greyish-white metal. When pure, it is malleable and "
        "ductile. Does not oxidize in air, insoluble in hydrochloric and "
        "nitric acid. Corroded by halogens, cyandies, sulphur and alkalis. "
        "Hydrogen and Oxygen react explosively in the presence of "
        "platinumpy. There are six stable isotopes and three radioisotopes, "
        "the most stable being Pt-193 with a half-life of 60 years. Platinum "
        "is used in jewelry, laboratory equipment, electrical contacts, "
        "dentistry, and anti-pollution devices in cars. PtCl2(NH3)2 is used "
        "to treat some forms of cancer. Platinum-Cobalt alloys have magnetic "
        "properties. It is also used in the definition of the Standard "
        "Hydrogen Electrode. Discovered by Antonio de Ulloa in South America "
        "in 1735. The name comes from the Spanish word platina which means "
        "silver. Platinum metal is generally not a health concern due to its "
        "unreactivity, however platinum compounds should be considered "
        "highly toxic."
    )
    elements['Au'].description = (
        "Gold is gold colored. It is the most malleable and ductile metal "
        "known. There is only one stable isotope of gold, and five "
        "radioisotopes of gold, Au-195 being the most stable with a "
        "half-life of 186 days. Gold is used as a monetary standard, in "
        "jewelry, dentistry, electronics. Au-198 is used in treating cancer "
        "and some other medical conditions. Gold has been known to exist as "
        "far back as 2600 BC. Gold comes from the Anglo-Saxon word gold. Its "
        "symbol, Au, comes from the Latin word aurum, which means gold. Gold "
        "is not particularly toxic, however it is known to cause damage to "
        "the liver and kidneys in some."
    )
    elements['Hg'].description = (
        "Heavy silvery liquid metallic element, belongs to the zinc group. "
        "Used in thermometers, barometers and other scientific apparatus. "
        "Less reactive than zinc and cadmium, does not displace hydrogen "
        "from acids. Forms a number of complexes and organomercury "
        "compounds."
    )
    elements['Tl'].description = (
        "Pure, unreacted thallium appears silvery-white and exhibits a "
        "metallic lustre. Upon reacting with air, it begins to turn "
        "bluish-grey and looks like lead. It is very malleable, and can be "
        "cut with a knife. There are two stable isotopes, and four "
        "radioisotopes, Tl-204 being the most stable with a half-life of "
        "3.78 years. Thallium sulphate was used as a rodenticide. Thallium "
        "sulphine's conductivity changes with exposure to infrared light, "
        "this gives it a use in infrared detectors. Discovered by Sir "
        "William Crookes via spectroscopy. Its name comes from the Greek "
        "word thallos, which means green twig. Thallium and its compounds "
        "are toxic and can cause cancer."
    )
    elements['Pb'].description = (
        "Heavy dull grey ductile metallic element, belongs to group 14. "
        "Used in building construction, lead-place accumulators, bullets and "
        "shot, and is part of solder, pewter, bearing metals, type metals "
        "and fusible alloys."
    )
    elements['Bi'].description = (
        "White crystalline metal with a pink tinge, belongs to group 15. "
        "Most diamagnetic of all metals and has the lowest thermal "
        "conductivity of all the elements except mercury. Lead-free bismuth "
        "compounds are used in cosmetics and medical procedures. Burns in "
        "the air and produces a blue flame. In 1753, C.G. Junine first "
        "demonstrated that it was different from lead."
    )
    elements['Po'].description = (
        "Rare radioactive metallic element, belongs to group 16 of the "
        "periodic table. Over 30 known isotopes exist, the most of all "
        "elements. Po-209 has a half-life of 103 years. Possible uses in "
        "heating spacecraft. Discovered by Marie Curie in 1898 in a sample "
        "of pitchblende."
    )
    elements['At'].description = (
        "Radioactive halogen element. Occurs naturally from uranium and "
        "thorium decay. At least 20 known isotopes. At-210, the most stable, "
        "has a half-life of 8.3 hours. Synthesized by nuclear bombardment in "
        "1940 by D.R. Corson, K.R. MacKenzie and E. Segre at the University "
        "of California."
    )
    elements['Rn'].description = (
        "Colorless radioactive gaseous element, belongs to the noble gases. "
        "Of the twenty known isotopes, the most stable is Rn-222 with a "
        "half-life of 3.8 days. Formed by the radioactive decay of "
        "Radium-226. Radon itself decays into Polonium. Used in "
        "radiotherapy. As a noble gas, it is effectively inert, though radon "
        "fluoride has been synthesized. First isolated in 1908 by Ramsey and "
        "Gray."
    )
    elements['Fr'].description = (
        "Radioactive element, belongs to group 1 of the periodic table. "
        "Found in uranium and thorium ores. The 22 known isotopes are all "
        "radioactive, with the most stable being Fr-223. Its existence was "
        "confirmed in 1939 by Marguerite Perey."
    )
    elements['Ra'].description = (
        "Radioactive metallic transuranic element, belongs to group 2 of "
        "the periodic table. Most stable isotope, Ra-226 has a half-life of "
        "1602 years, which decays into radon. Isolated from pitchblende in "
        "1898 Marie and Pierre Curie."
    )
    elements['Ac'].description = (
        "Silvery radioactive metallic element, belongs to group 3 of the "
        "periodic table. The most stable isotope, Ac-227, has a half-life of "
        "217 years. Ac-228 (half-life of 6.13 hours) also occurs in nature. "
        "There are 22 other artificial isotopes, all radioactive and having "
        "very short half-lives. Chemistry similar to lanthanumpy. Used as a "
        "source of alpha particles. Discovered by A. Debierne in 1899."
    )
    elements['Th'].description = (
        "Grey radioactive metallic element. Belongs to actinoids. Found in "
        "monazite sand in Brazil, India and the US. Thorium-232 has a "
        "half-life of 1.39x10^10 years. Can be used as a nuclear fuel for "
        "breeder reactors. Thorium-232 captures slow Neutrons and breeds "
        "uranium-233. Discovered by Jons J. Berzelius in 1829."
    )
    elements['Pa'].description = (
        "Radioactive metallic element, belongs to the actinoids. The most "
        "stable isotope, Pa-231 has a half-life of 2.43*10^4 years. At least "
        "10 other radioactive isotopes are known. No practical applications "
        "are known. Discovered in 1917 by Lise Meitner and Otto Hahn."
    )
    elements['U'].description = (
        "White radioactive metallic element belonging to the actinoids. "
        "Three natural isotopes, U-238, U-235 and U-234. Uranium-235 is used "
        "as the fuel for nuclear reactors and weapons. Discovered by Martin "
        "H. Klaproth in 1789."
    )
    elements['Np'].description = (
        "Radioactive metallic transuranic element, belongs to the "
        "actinoids. Np-237, the most stable isotope, has a half-life of "
        "2.2*10^6 years and is a by product of nuclear reactors. The other "
        "known isotopes have mass numbers 229 through 236, and 238 through "
        "241. Np-236 has a half-life of 5*10^3 years. First produced by "
        "Edwin M. McMillan and P.H. Abelson in 1940."
    )
    elements['Pu'].description = (
        "Dense silvery radioactive metallic transuranic element, belongs to "
        "the actinoids. Pu-244 is the most stable isotope with a half-life "
        "of 7.6*10^7 years. Thirteen isotopes are known. Pu-239 is the most "
        "important, it undergoes nuclear fission with slow neutrons and is "
        "hence important to nuclear weapons and reactors. Plutonium "
        "production is monitored down to the gram to prevent military "
        "misuse. First produced by Gleen T. Seaborg, Edwin M. McMillan, J.W. "
        "Kennedy and A.C. Wahl in 1940."
    )
    elements['Am'].description = (
        "Radioactive metallic transuranic element, belongs to the "
        "actinoids. Ten known isotopes. Am-243 is the most stable isotope, "
        "with a half-life of 7.95*10^3 years. Discovered by Glenn T. Seaborg "
        "and associates in 1945, it was obtained by bombarding Uranium-238 "
        "with alpha particles."
    )
    elements['Cm'].description = (
        "Radioactive metallic transuranic element. Belongs to actinoid "
        "series. Nine known isotopes, Cm-247 has a half-life of 1.64*10^7 "
        "years. First identified by Glenn T. Seaborg and associates in 1944, "
        "first produced by L.B. Werner and I. Perlman in 1947 by bombarding "
        "americium-241 with Neutrons. Named for Marie Curie."
    )
    elements['Bk'].description = (
        "Radioactive metallic transuranic element. Belongs to actinoid "
        "series. Eight known isotopes, the most common Bk-247, has a "
        "half-life of 1.4*10^3 years. First produced by Glenn T. Seaborg and "
        "associates in 1949 by bombarding americium-241 with alpha "
        "particles."
    )
    elements['Cf'].description = (
        "Radioactive metallic transuranic element. Belongs to actinoid "
        "series. Cf-251 has a half life of about 700 years. Nine isotopes "
        "are known. Cf-252 is an intense Neutron source, which makes it an "
        "intense Neutron source and gives it a use in Neutron activation "
        "analysis and a possible use as a radiation source in medicine. "
        "First produced by Glenn T. Seaborg and associates in 1950."
    )
    elements['Es'].description = (
        "Appearance is unknown, however it is most probably metallic and "
        "silver or gray in color. Radioactive metallic transuranic element "
        "belonging to the actinoids. Es-254 has the longest half-life of the "
        "eleven known isotopes at 270 days. First identified by Albert "
        "Ghiorso and associates in the debris of the 1952 hydrogen bomb "
        "explosion. In 1961 the first microgram quantities of Es-232 were "
        "separated. While einsteinium never exists naturally, if a "
        "sufficient amount was assembled, it would pose a radiation hazard."
    )
    elements['Fm'].description = (
        "Radioactive metallic transuranic element, belongs to the "
        "actinoids. Ten known isotopes, most stable is Fm-257 with a "
        "half-life of 10 days. First identified by Albert Ghiorso and "
        "associates in the debris of the first hydrogen-bomb explosion in "
        "1952."
    )
    elements['Md'].description = (
        "Radioactive metallic transuranic element. Belongs to the actinoid "
        "series. Only known isotope, Md-256 has a half-life of 1.3 hours. "
        "First identified by Glenn T. Seaborg, Albert Ghiorso and associates "
        "in 1955. Alternative name Unnilunium has been proposed. Named after "
        "the 'inventor' of the periodic table, Dmitri Mendeleev."
    )
    elements['No'].description = (
        "Radioactive metallic transuranic element, belongs to the "
        "actinoids. Seven known isotopes exist, the most stable being No-254 "
        "with a half-life of 255 seconds. First identified with certainty by "
        "Albert Ghiorso and Glenn T. Seaborg in 1966. Unnilbium has been "
        "proposed as an alternative name."
    )
    elements['Lr'].description = (
        "Appearance unknown, however it is most likely silvery-white or "
        "grey and metallic. Lawrencium is a synthetic rare-earth metal. "
        "There are eight known radioisotopes, the most stable being Lr-262 "
        "with a half-life of 3.6 hours. Due to the short half-life of "
        "lawrencium, and its radioactivity, there are no known uses for it. "
        "Identified by Albert Ghiorso in 1961 at Berkeley. It was produced "
        "by bombarding californium with boron ions. The name is temporary "
        "IUPAC nomenclature, the origin of the name comes from Ernest O. "
        "Lawrence, the inventor of the cyclotron. If sufficient amounts of "
        "lawrencium were produced, it would pose a radiation hazard."
    )
    elements['Rf'].description = (
        "Radioactive transactinide element. Expected to have similar "
        "chemical properties to those displayed by hafnium. Rf-260 was "
        "discovered by the Joint Nuclear Research Institute at Dubna "
        "(U.S.S.R.) in 1964. Researchers at Berkeley discovered Unq-257 and "
        "Unq-258 in 1964."
    )
    elements['Db'].description = (
        "Also known as Hahnium, Ha. Radioactive transactinide element. "
        "Half-life of 1.6s. Discovered in 1970 by Berkeley researchers. So "
        "far, seven isotopes have been discovered."
    )
    elements['Sg'].description = (
        "Half-life of 0.9 +/- 0.2 s. Discovered by the Joint Institute for "
        "Nuclear Research at Dubna (U.S.S.R.) in June of 1974. Its existence "
        "was confirmed by the Lawrence Berkeley Laboratory and Livermore "
        "National Laboratory in September of 1974."
    )
    elements['Bh'].description = (
        "Radioactive transition metal. Half-life of approximately 1/500 s. "
        "Discovered by the Joint Institute for Nuclear Research at Dubna "
        "(U.S.S.R.) in 1976. Confirmed by West German physicists at the "
        "Heavy Ion Research Laboratory at Darmstadt."
    )
    elements['Hs'].description = (
        "Radioactive transition metal first synthesized in 1984 by a German "
        "research team led by Peter Armbruster and Gottfried Muenzenberg at "
        "the Institute for Heavy Ion Research at Darmstadt."
    )
    elements['Mt'].description = (
        "Half-life of approximately 5 ms. The creation of this element "
        "demonstrated that fusion techniques could indeed be used to make "
        "new, heavy nuclei. Made and identified by physicists of the Heavy "
        "Ion Research Laboratory, Darmstadt, West Germany in 1982. Named in "
        "honor of Lise Meitner, the Austrian physicist."
    )

    if symbol is not None:
        return elements[symbol].description


if __name__ == '__main__':

    try:
        from .elements import ELEMENTS, word_wrap
    except ImportError:
        from elements import ELEMENTS, word_wrap

    elements_descriptions(ELEMENTS)

    for ele in ELEMENTS:
        descr = word_wrap(
            ele.description, linelen=68, indent=0, joinstr=' "\n        "'
        )
        descr = (
            f"    elements['{ele.symbol}'].description = "
            f"(\n        \"{descr}\"    \n    )"
        )
        print(descr)
